#include "ac33mx128_pcu.h"
#include "ac33mx128_scu.h"
#include "ac33mx128_dmac.h"
#include "ac33mx128_gpio.h"
#include "ac33mx128_spi.h"
#include "debug_frmwrk.h"
#include "ac33mx128_libcfg.h"

#define TEST_SINGLE_POLLING // spi master polling : single data 
//#define TEST_SINGLE_INTERRUPT // spi master interrupt : single data 
//#define TEST_MULTI_INTERRUPT // spi master interrupt : multi data 10 bytes
//#define TEST_MULTI_DMA_INTERRUPT // spi master dma : multi data 10 bytes


// system default I/O setting
void PCU_Init(void);

const uint8_t menu[] =
"************************************************\n\r"
" SPI master demo \n\r"
"\t - MCU: AC33M8128 \n\r"
"\t - Core: ARM Cortex-M3 \n\r"
"\t - Communicate via: UART0 - 115200 bps \n\r"
"  \n\r"
"************************************************\n\r";
/**********************************************************************
 * @brief		Print menu
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void print_menu(void)
{
	_DBG(menu);
}

#ifdef TEST_SINGLE_POLLING // spi master polling mode 
// for this test, MOSI and MISO must be connected.  
void mainloop(void)
{
	uint32_t rxdata;
	uint32_t tmp;
	uint32_t i;
	SPI_CFG_Type SP0_config;
	
	PCU_ConfigureFunction(PCD, 9, PD9_MUX_PD9);
	PCU_SetDirection(PCD, 9, PUSH_PULL_OUTPUT);
	PCU_ConfigurePullup(PCD, 9, DISABLE);		
	GPIO_SetValue(PD, _BIT(9));
	
	print_menu();
	
	SP0_config.Databit = SPI_DS_8BITS;
	SP0_config.CPHA = SPI_CPHA_LO;
	SP0_config.CPOL = SPI_CPOL_LO;
	SP0_config.DataDir = SPI_MSB_FIRST;
	SP0_config.Mode = SPI_MASTER_MODE;
	SP0_config.BaudRate = 71; // PCLK / (71+1) 
	
	SPI_Init(SP0, &SP0_config);
	SPI_Cmd(SP0, ENABLE);  // SPI0 enable 

	_DBG_(" ");
	i=0x30;
	while(1){
		while((SPI_GetStatus(SP0) & SPI_STAT_TXBUF_EMPTY) != SPI_STAT_TXBUF_EMPTY); // check tx buffer empty 
		SPI_SendData(SP0, i);

		while((SPI_GetStatus(SP0) & SPI_STAT_RXBUF_READY) != SPI_STAT_RXBUF_READY); // check rx buffer full	
		rxdata = SPI_ReceiveData(SP0);
		
		if (rxdata == i) GPIO_SetValue(PD, _BIT(9));
		else GPIO_ClearValue(PD, _BIT(9));
		
		i++; i&=(0xff);
		for (tmp=0;tmp <1000000; tmp++);	
	}
}
#endif

#ifdef TEST_SINGLE_INTERRUPT // spi master interrupt : single data 
// for this test, MOSI and MISO must be connected.  

uint32_t fflag;
volatile uint32_t rxdata, txdata;

void SPI0_IRQHandler(void)
{
	uint32_t status;
	
	status = SPI_GetStatus(SP0);
	if ( (status & SPI_STAT_TXBUF_EMPTY) && (SPI_GetControl(SP0) & SPI_INTCFG_TXIE)){ // tx buffer empty
		SPI_IntConfig(SP0, SPI_INTCFG_TXIE, DISABLE);
		SPI_SendData(SP0, txdata);		
	}
	
	if ( (status & SPI_STAT_RXBUF_READY) && (SPI_GetControl(SP0) & SPI_INTCFG_RXIE)){ // rx buffer full		
		rxdata = SPI_ReceiveData(SP0);
		fflag=1;
	}
}


void mainloop(void)
{
	uint32_t tmp;
	uint32_t i;
	SPI_CFG_Type SP0_config;
	
	PCU_ConfigureFunction(PCD, 9, PD9_MUX_PD9);
	PCU_SetDirection(PCD, 9, PUSH_PULL_OUTPUT);
	PCU_ConfigurePullup(PCD, 9, DISABLE);		
	GPIO_SetValue(PD, _BIT(9));
	
	print_menu();
	
	/* SPI0 enable */
	SP0_config.Databit = SPI_DS_8BITS;
	SP0_config.CPHA = SPI_CPHA_LO;
	SP0_config.CPOL = SPI_CPOL_LO;
	SP0_config.DataDir = SPI_MSB_FIRST;
	SP0_config.Mode = SPI_MASTER_MODE;
	SP0_config.BaudRate = 71; // PCLK / (71+1) 
	SPI_Init(SP0, &SP0_config);
	
	SPI_IntConfig(SP0, SPI_INTCFG_RXIE, ENABLE); // rx interrupt enable 
	SPI_Cmd(SP0, ENABLE);  // SPI0 enable 

    /* preemption = 1, sub-priority = 1 */
    NVIC_SetPriority(SPI0_IRQn, ((0x01<<1)|0x01));		
	NVIC_EnableIRQ(SPI0_IRQn);

	__enable_irq();
	
	_DBG_(" ");
	i=0x30;
	while(1){
		txdata = i;
		fflag=0;
		SPI_IntConfig(SP0, SPI_INTCFG_TXIE, ENABLE);	

		while(fflag == 0); // check rx buffer full 

		if (rxdata == txdata)
			GPIO_SetValue(PD, _BIT(9));
		else 
			GPIO_ClearValue(PD, _BIT(9));
		
		i++;
		i&=(0xff);
		for (tmp=0;tmp <1000000; tmp++);	
	}
}
#endif

#ifdef TEST_MULTI_INTERRUPT // spi master interrupt : multi data 
// for this test, MOSI and MISO must be connected. 

uint32_t fflag;

volatile uint32_t rxdata[10], txdata[10];
uint32_t cnt, cntrx;


void SPI0_IRQHandler(void)
{
	uint32_t status;
	
	status=SPI_GetStatus(SP0);
	if ( (status & SPI_STAT_TXBUF_EMPTY) && (SPI_GetControl(SP0) & SPI_INTCFG_TXIE)){ // tx buffer empty
		SPI_SendData(SP0, txdata[cnt]);	
		cnt++;
		if (cnt==10){
			SPI_IntConfig(SP0, SPI_INTCFG_TXIE, DISABLE); //TXIE disable
		}
	}

	if ( (status & SPI_STAT_RXBUF_READY) && (SPI_GetControl(SP0) & SPI_INTCFG_RXIE)){ // rx buffer full		
		rxdata[cntrx] = SPI_ReceiveData(SP0);
		cntrx++;
		if (cntrx==10){
			fflag=1;
		}
	}
}


void mainloop(void)
{
	uint32_t tmp;
	uint32_t i;
	uint32_t stat;
	SPI_CFG_Type SP0_config;
	
	PCU_ConfigureFunction(PCD, 9, PD9_MUX_PD9);
	PCU_SetDirection(PCD, 9, PUSH_PULL_OUTPUT);
	PCU_ConfigurePullup(PCD, 9, DISABLE);		
	GPIO_SetValue(PD, _BIT(9));
	
	print_menu();

	/* SPI0 enable */
	SP0_config.Databit = SPI_DS_8BITS;
	SP0_config.CPHA = SPI_CPHA_LO;
	SP0_config.CPOL = SPI_CPOL_LO;
	SP0_config.DataDir = SPI_MSB_FIRST;
	SP0_config.Mode = SPI_MASTER_MODE;
	SP0_config.BaudRate = 71; // PCLK / (71+1) 
	SPI_Init(SP0, &SP0_config);
	
	SPI_DelayConfig(SP0, 1, 10, 1);

	SPI_IntConfig(SP0, SPI_INTCFG_RXIE, ENABLE); // rx interrupt enable 
	SPI_Cmd(SP0, ENABLE);  // SPI0 enable 

    /* preemption = 1, sub-priority = 1 */
    NVIC_SetPriority(SPI0_IRQn, ((0x01<<1)|0x01));		
	NVIC_EnableIRQ(SPI0_IRQn);

	__enable_irq();
	
	_DBG_(" ");
	while(1){
		for(i=0;i<10;i++){
			txdata[i]=i+0x30;
			rxdata[i]=0;
		}
		cnt=0; cntrx=0;
		fflag=0;
		SPI_IntConfig(SP0, SPI_INTCFG_TXIE, ENABLE); 

		while(fflag == 0); // check rx buffer full 

		stat=0;
		for (i=0;i<10;i++){
			if (rxdata[i] != txdata[i])
				stat=1;			
		}
		
		if (stat == 1)
			GPIO_SetValue(PD, _BIT(9));
		else 
			GPIO_ClearValue(PD, _BIT(9));
		
		for (tmp=0;tmp <1000000; tmp++);	
	}
}
#endif


#ifdef TEST_MULTI_DMA_INTERRUPT // spi master dma : multi data 
// for this test, MOSI and MISO must be connected. 

uint32_t fflag;

volatile uint8_t rxdata[10], txdata[10];
uint32_t cnt, cntrx;


void SPI0_IRQHandler(void)
{
	uint32_t status;
	
	status=SPI_GetStatus(SP0);
	if ( (status & SPI_STAT_TXDMA_DONE) && (SPI_GetControl(SP0) & SPI_INTCFG_TXDIE)){ // tx dma done		
		SPI_ClearPending(SP0, SPI_STAT_TXDMA_DONE);
		DMAC_ChCmd(DC9, DISABLE);	
	}

	if ( (status & SPI_STAT_RXDMA_DONE) && (SPI_GetControl(SP0) & SPI_INTCFG_RXDIE)){ // rx dma done	
		SPI_ClearPending(SP0, SPI_STAT_RXDMA_DONE);		
		DMAC_ChCmd(DC8, DISABLE);		
		fflag=1;
	}
}


void mainloop(void)
{
	uint32_t tmp;
	uint32_t i;
	uint32_t stat;
	SPI_CFG_Type SP0_config;	
	DMAC_Ch_CFG_Type DMACSPI0Rx;
	DMAC_Ch_CFG_Type DMACSPI0Tx;
	
	PCU_ConfigureFunction(PCD, 9, PD9_MUX_PD9);
	PCU_SetDirection(PCD, 9, PUSH_PULL_OUTPUT);
	PCU_ConfigurePullup(PCD, 9, DISABLE);		
	GPIO_SetValue(PD, _BIT(9));
	
	print_menu();
	
	/* SPI0 enable */
	SP0_config.Databit = SPI_DS_8BITS;
	SP0_config.CPHA = SPI_CPHA_LO;
	SP0_config.CPOL = SPI_CPOL_LO;
	SP0_config.DataDir = SPI_MSB_FIRST;
	SP0_config.Mode = SPI_MASTER_MODE;
	SP0_config.BaudRate = 71; // PCLK / (71+1) 
	SPI_Init(SP0, &SP0_config);

	SPI_DelayConfig(SP0, 1, 20, 1);

	SPI_IntConfig(SP0, (SPI_INTCFG_TXDIE | SPI_INTCFG_RXDIE), ENABLE); // rx interrupt enable 
	SPI_Cmd(SP0, ENABLE);  // SPI0 enable 

    /* preemption = 1, sub-priority = 1 */
    NVIC_SetPriority(SPI0_IRQn, ((0x01<<1)|0x01));		
	NVIC_EnableIRQ(SPI0_IRQn);

	__enable_irq();
	
	_DBG_(" ");
	while(1){
		for(i=0;i<10;i++){
			txdata[i]=i+0x30;
			rxdata[i]=0;
		}
		DMACSPI0Rx.TransferCnt = 10;
		DMACSPI0Rx.TransferWidth = DMAC_WIDTH_BYTE;
		DMACSPI0Rx.MemAddr = (uint32_t)(&rxdata[0]);
		DMAC_Setup(DC8, &DMACSPI0Rx);

		DMAC_ChCmd(DC8, ENABLE);

		DMACSPI0Tx.TransferCnt = 10;
		DMACSPI0Tx.TransferWidth = DMAC_WIDTH_BYTE;
		DMACSPI0Tx.MemAddr = (uint32_t)(&txdata[0]);	
		DMAC_Setup(DC9, &DMACSPI0Tx);	
		
		fflag = 0;	
		DMAC_ChCmd(DC9, ENABLE);
		
		while(fflag == 0); // check rx buffer full 

		stat=0;
		for (i=0;i<10;i++){
			if (rxdata[i] != txdata[i])
				stat=1;			
		}
		
		if (stat == 1)
			GPIO_SetValue(PD, _BIT(9));
		else 
			GPIO_ClearValue(PD, _BIT(9));
		
		for (tmp=0;tmp <1000000; tmp++);	
	}
}
#endif


int main(void)
{
	SystemInit();
	PCU_Init(); // all port control setting  !!You have to set PORT function!!
	SCU_ClockInit(); // MCLK=72Mhz(PLL) setting 
	DMAC_Init(); // DMAC peri enable 
	debug_frmwrk_init();
	
	mainloop();
	
	return (0);
}


void PCU_Init(void)
{
//Peripheral Enable Register 1  0:Disable, 1:Enable	
	SCU->PER1=SCU->PER1  
	| (1<<11)   // GPIOD
	| (1<<10)   // GPIOC
	| (1<<9)   // GPIOB
	| (1<<8)   // GPIOA
		; 		
//Peripheral Clock Enable Register 1 0:Disable, 1:Enable	
	SCU->PCER1=SCU->PCER1
	| (1<<11)   // GPIOD
	| (1<<10)   // GPIOC
	| (1<<9)   // GPIOB
	| (1<<8)   // GPIOA
		; 	
	
 	PORT_ACCESS_EN();  // enable writing permittion of ALL PCU register

	PCA->MR = 0 // PinMux setting (00b,01b,10b,11b)
	| (1UL<<30)   // 0:PA15, 1:MISO0,     2:T7C, 3:AN15
	| (1UL<<28)   // 0:PA14, 1:MOSI0,            3:AN14
	| (1UL<<26)   // 0:PA13, 1:SCK0,              3:AN13
	| (1UL<<24)   // 0:PA12, 1:SS0,         2:AD2I,     3:AN12
	| (0UL<<22)   // 0:PA11, 1:TRACED2,        3:AN11
	| (0UL<<20)   // 0:PA10, 1:TRACED1, 2:AD2O, 3:AN10
	| (0UL<<18)   // 0:PA9,  1:TRACED0, 2:AD1O, 3:AN9
	| (0UL<<16)   // 0:PA8, 1:TRACECLK, 2:AD0O, 3:AN8
	| (0UL<<14)   // 0:PA7, 1:TRACED3,  2:T3O, 3:AN7_CREF1
	| (0UL<<12)   // 0:PA6,                     2:T2O, 3:AN6_CREF0
	| (0UL<<10)   // 0:PA5,                     2:T1O, 3:AN5
	| (0UL<<8)   // 0:PA4,                       2:T0O, 3:AN4
	| (0UL<<6)   // 0:PA3,                            3:AN3_CP3
	| (0UL<<4)   // 0:PA2,                            3:AN2_CP2	
	| (0UL<<2)   // 0:PA1,                            3:AN1_CP1	
	| (0UL<<0)   // 0:PA0,                            3:AN0_CP0		
		; 
	PCA->CR = 0 // 0xFFFFFFFF(reset value) Input, output, open drain, analog
	| (2UL<<30)   // P15 0:Push-pull output, 1:Open-drain output, 2:Input, 3:Analog
	| (0UL<<28)   // P14
	| (0UL<<26)   // P13
	| (0UL<<24)   // P12
	| (0UL<<22)   // P11
	| (0UL<<20)   // P10
	| (0UL<<18)   // P9
	| (0UL<<16)   // P8
	| (0UL<<14)   // P7
	| (0UL<<12)   // P6
	| (0UL<<10)   // P5
	| (0UL<<8)   // P4
	| (0UL<<6)   // P3
	| (0UL<<4)   // P2
	| (0UL<<2)   // P1
	| (0UL<<0)   // P0	
		; 	
	PCA->PCR = 0 // pullup (0,1)
	| (0<<15)   // P15 0:Disable pull-up resistor, 1:Enable pull-up resister
	| (0<<14)   // P14
	| (0<<13)   // P13
	| (0<<12)   // P12
	| (0<<11)   // P11
	| (0<<10)   // P10
	| (0<<9)   // P9
	| (0<<8)   // P8
	| (0<<7)   // P7
	| (0<<6)   // P6
	| (0<<5)   // P5
	| (0<<4)   // P4
	| (0<<3)   // P3
	| (0<<2)   // P2
	| (0<<1)   // P1
	| (0<<0)   // P0	
		; 		
	
	PCB->MR = 0 // PinMux setting (00b,01b,10b,11b)
	| (0UL<<30)   // 0:PB15, 1:MP1WL
	| (0UL<<28)   // 0:PB14, 1:MP1WH
	| (0UL<<26)   // 0:PB13, 1:MP1VL
	| (0UL<<24)   // 0:PB12, 1:MP1VH
	| (0UL<<22)   // 0:PB11, 1:MP1UL
	| (0UL<<20)   // 0:PB10, 1:MP1UH
	| (0UL<<18)   // 0:PB9,  1:OVIN1, 2:TXD3
	| (0UL<<16)   // 0:PB8, 1:PRTIN1, 2:RXD3
	| (0UL<<14)   // 0:PB7, 1:OVIN0, 2:STBYO
	| (0UL<<12)   // 0:PB6, 1:PRTIN0, 2:WDTO
	| (0UL<<10)   // 0:PB5, 1:MP0WL, 2:T9O
	| (0UL<<8)   // 0:PB4, 1:MP0WH, 2:T9C
	| (0UL<<6)   // 0:PB3, 1:MP0VL
	| (0UL<<4)   // 0:PB2, 1:MP0VH
	| (0UL<<2)   // 0:PB1, 1:MP0UL
	| (0UL<<0)   // 0:PB0, 1:MP0UH 
		;	
	PCB->CR = 0   // 0xFFFFFFFF(reset value) Input, output, open drain, analog
	| (0UL<<30)   // P15 0:Push-pull output, 1:Open-drain output, 2:Input, 3:Analog
	| (0UL<<28)   // P14
	| (0UL<<26)   // P13
	| (0UL<<24)   // P12
	| (0UL<<22)   // P11
	| (0UL<<20)   // P10
	| (0UL<<18)   // P9
	| (0UL<<16)   // P8
	| (0UL<<14)   // P7
	| (0UL<<12)   // P6
	| (0UL<<10)   // P5
	| (0UL<<8)   // P4
	| (0UL<<6)   // P3
	| (0UL<<4)   // P2
	| (0UL<<2)   // P1
	| (0UL<<0)   // P0	
		; 	
	PCB->PCR = 0 // pullup (0,1)
	| (0<<15)   // P15 0:Disable pull-up resistor, 1:Enable pull-up resister
	| (0<<14)   // P14
	| (0<<13)   // P13
	| (0<<12)   // P12
	| (0<<11)   // P11
	| (0<<10)   // P10
	| (0<<9)   // P9
	| (0<<8)   // P8
	| (0<<7)   // P7
	| (0<<6)   // P6
	| (0<<5)   // P5
	| (0<<4)   // P4
	| (0<<3)   // P3
	| (0<<2)   // P2
	| (0<<1)   // P1
	| (0<<0)   // P0	
		; 
	
	PCC->MR = 0 //0x00100155(reset value) // PinMux setting (00b,01b,10b,11b)
	| (1UL<<30)   // 0:PC15, 1:TXD0, 2:MOSI0
	| (1UL<<28)   // 0:PC14, 1:RXD0, 2:MISO0, 3:VMARGIN
	| (1UL<<26)   // 0:PC13, 1:XOUT
	| (1UL<<24)   // 0:PC12, 1:XIN
	| (0UL<<22)   // 0:PC11,            2:T8C
	| (1UL<<20)   // 0:PC10, 1:nRESET,
	| (0UL<<18)   // 0:PC9,  1:CLKO, 2:P8O
	| (0UL<<16)   // 0:PC8, 1:SDA0
	| (0UL<<14)   // 0:PC7, 1:SCL0, 2:T3C
	| (0UL<<12)   // 0:PC6, 1:TXD1, 2:T2C_PHZ
	| (0UL<<10)   // 0:PC5, 1:RXD1, 2:T1C_PHB
	| (1UL<<8)   // 0:PC4, 1:nTRST, 2:T0C_PHA
	| (1UL<<6)   // 0:PC3, 1:TDI
	| (1UL<<4)   // 0:PC2, 1:TDO_SWO
	| (1UL<<2)   // 0:PC1, 1:TMS_SWDIO
	| (1UL<<0)   // 0:PC0, 1:TCK_SWCLK 
		;		
	PCC->CR = 0   // 0xFFAFFE8A(reset value) Input, output, open drain, analog
	| (0UL<<30)   // P15 0:Push-pull output, 1:Open-drain output, 2:Input, 3:Analog
	| (2UL<<28)   // P14
	| (3UL<<26)   // P13  if XOUT, it must be Analog 
	| (3UL<<24)   // P12  if XIN, it must be Analog
	| (0UL<<22)   // P11
	| (2UL<<20)   // P10
	| (0UL<<18)   // P9
	| (0UL<<16)   // P8
	| (0UL<<14)   // P7
	| (0UL<<12)   // P6
	| (0UL<<10)   // P5
	| (2UL<<8)   // P4
	| (2UL<<6)   // P3
	| (2UL<<4)   // P2
	| (2UL<<2)   // P1
	| (2UL<<0)   // P0	
		; 	
	PCC->PCR = 0 // pullup (0,1)
	| (0<<15)   // P15 0:Disable pull-up resistor, 1:Enable pull-up resister
	| (0<<14)   // P14
	| (0<<13)   // P13
	| (0<<12)   // P12
	| (0<<11)   // P11
	| (0<<10)   // P10
	| (0<<9)   // P9
	| (0<<8)   // P8
	| (0<<7)   // P7
	| (0<<6)   // P6
	| (0<<5)   // P5
	| (0<<4)   // P4
	| (0<<3)   // P3
	| (0<<2)   // P2
	| (0<<1)   // P1
	| (0<<0)   // P0	
		; 

	PCD->MR = 0 // PinMux setting (00b,01b,10b,11b)
	| (0UL<<30)   // 0:PD15, 1:AD2EOC
	| (0UL<<28)   // 0:PD14, 1:AD2SOC
	| (0UL<<26)   // 0:PD13, 1:AD1EOC, 2:T3C
	| (0UL<<24)   // 0:PD12, 1:AD1SOC, 2:T2C_PHZ
	| (0UL<<22)   // 0:PD11, 1:AD0EOC, 2:T1C_PHB
	| (0UL<<20)   // 0:PD10, 1:AD0SOC, 2:T0C_PHA
	| (0UL<<18)   // 0:PD9,                     2:STBYO
	| (0UL<<16)   // 0:PD8,                     2:WDTO
	| (0UL<<14)   // 0:PD7, 1:RXD2, 2:AD1I
	| (0UL<<12)   // 0:PD6, 1:TXD2, 2:AD0I
	| (0UL<<10)   // 0:PD5, 1:SDA1
	| (0UL<<8)   // 0:PD4, 1:SCL1
	| (1UL<<6)   // 0:PD3, 1:MISO1
	| (1UL<<4)   // 0:PD2, 1:MOSI1
	| (1UL<<2)   // 0:PD1, 1:SCK1
	| (1UL<<0)   // 0:PD0, 1:SS1
		;		
	PCD->CR = 0   // 0xFFFFFFFF(reset value) Input, output, open drain, analog
	| (0UL<<30)   // P15 0:Push-pull output, 1:Open-drain output, 2:Input, 3:Analog
	| (0UL<<28)   // P14
	| (0UL<<26)   // P13
	| (0UL<<24)   // P12
	| (0UL<<22)   // P11
	| (0UL<<20)   // P10
	| (0UL<<18)   // P9
	| (0UL<<16)   // P8
	| (0UL<<14)   // P7
	| (0UL<<12)   // P6
	| (0UL<<10)   // P5
	| (0UL<<8)   // P4
	| (2UL<<6)   // P3
	| (0UL<<4)   // P2
	| (0UL<<2)   // P1
	| (0UL<<0)   // P0	
		; 	
	PCD->PCR = 0 // pullup (0,1)
	| (0<<15)   // P15 0:Disable pull-up resistor, 1:Enable pull-up resister
	| (0<<14)   // P14
	| (0<<13)   // P13
	| (0<<12)   // P12
	| (0<<11)   // P11
	| (0<<10)   // P10
	| (0<<9)   // P9
	| (0<<8)   // P8
	| (0<<7)   // P7
	| (0<<6)   // P6
	| (0<<5)   // P5
	| (0<<4)   // P4
	| (0<<3)   // P3
	| (0<<2)   // P2
	| (0<<1)   // P1
	| (0<<0)   // P0	
		; 
	PORT_ACCESS_DIS();  // disable writing permittion of ALL PCU register 	
}
